package it.sauronsoftware.kickstart;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.PrintWriter;
import java.io.StringWriter;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;

/**
 * La finestra di dialog che mostra i dettagli di un errore.
 * 
 * @author Carlo Pelliccia
 */
class ErrorLogDialog extends JDialog {

	private static final long serialVersionUID = 1L;

	/**
	 * L'eccezione mostrata.
	 */
	private Exception exception = null;

	/**
	 * Crea la finestra di dialogo.
	 * 
	 * @param owner
	 *            La finestra proprietaria
	 * @param exception
	 *            L'eccezione da mostrare.
	 */
	public ErrorLogDialog(Kickstart owner, Exception exception) {
		super(owner, true);
		this.exception = exception;
		initialize();
		pack();
		setResizable(false);
		Point ownerLocation = owner.getLocation();
		Dimension ownerSize = owner.getSize();
		Point dialogLocation = new Point();
		Dimension dialogSize = getSize();
		dialogLocation.x = ownerLocation.x
				+ ((ownerSize.width - dialogSize.width) / 2);
		dialogLocation.y = ownerLocation.y
				+ ((ownerSize.height - dialogSize.height) / 2);
		setLocation(dialogLocation);
		setDefaultCloseOperation(DISPOSE_ON_CLOSE);
	}

	/**
	 * Inizializza la GUI.
	 */
	private void initialize() {
		LanguagePack pack = LanguagePack.getInstance();
		setTitle(pack.getMessage("ERRORLOGDIALOG_TITLE"));
		JButton close = new JButton(pack
				.getMessage("ERRORLOGDIALOG_CLOSE_CAPTION"));
		close.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				dispose();
			}
		});
		JTextArea logArea = new JTextArea(getExceptionText());
		logArea.setEditable(false);
		JScrollPane scrollPane = new JScrollPane(logArea);
		scrollPane.setPreferredSize(new Dimension(500, 250));
		JPanel closePanel = new JPanel();
		closePanel.setLayout(new FlowLayout(FlowLayout.CENTER));
		closePanel.add(close);
		JPanel contentPane = new JPanel();
		contentPane.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));
		contentPane.setLayout(new BorderLayout(3, 3));
		contentPane.add(scrollPane, BorderLayout.CENTER);
		contentPane.add(closePanel, BorderLayout.SOUTH);
		setContentPane(contentPane);
	}

	/**
	 * Stampa su stringa lo stack trace dell'eccezione.
	 * 
	 * @return Lo stack trace dell'eccezione.
	 */
	private String getExceptionText() {
		StringWriter stringWriter = new StringWriter();
		PrintWriter printWriter = new PrintWriter(stringWriter);
		exception.printStackTrace(printWriter);
		printWriter.close();
		return stringWriter.toString();
	}

}
