package it.sauronsoftware.kickstart;

import it.sauronsoftware.kickstart.generator.EXEGenerator;
import it.sauronsoftware.kickstart.generator.GenerationListener;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Point;

import javax.swing.BorderFactory;
import javax.swing.JDialog;
import javax.swing.JPanel;
import javax.swing.JProgressBar;

/**
 * La finestra di dialog che esegue e controlla la routine di generazione
 * dell'EXE.
 * 
 * @author Carlo Pelliccia
 */
class GenerationDialog extends JDialog {

	private static final long serialVersionUID = 1L;

	/**
	 * La routine di generazione da eseguire.
	 */
	private EXEGenerator generator = null;

	/**
	 * La barra di progresso.
	 */
	private JProgressBar progressBar = null;

	/**
	 * Eventuale eccezione di insuccesso della routine.
	 */
	private Exception exception = null;

	/**
	 * Crea la finestra di dialogo.
	 * 
	 * @param owner
	 *            La finestra proprietaria
	 * @param generator
	 *            La routine di generazione da eseguire.
	 */
	public GenerationDialog(Kickstart owner, EXEGenerator generator) {
		super(owner, true);
		this.generator = generator;
		initialize();
		pack();
		setResizable(false);
		Point ownerLocation = owner.getLocation();
		Dimension ownerSize = owner.getSize();
		Point dialogLocation = new Point();
		Dimension dialogSize = getSize();
		dialogLocation.x = ownerLocation.x
				+ ((ownerSize.width - dialogSize.width) / 2);
		dialogLocation.y = ownerLocation.y
				+ ((ownerSize.height - dialogSize.height) / 2);
		setLocation(dialogLocation);
		setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
	}

	/**
	 * Inizializza la GUI.
	 */
	private void initialize() {
		LanguagePack pack = LanguagePack.getInstance();
		setTitle(pack.getMessage("GENERATORDIALOG_TITLE"));
		progressBar = new JProgressBar(0, 100);
		Dimension preferredSize = progressBar.getPreferredSize();
		preferredSize.width = 200;
		progressBar.setPreferredSize(preferredSize);
		JPanel contentPane = new JPanel();
		contentPane.setBorder(BorderFactory.createEmptyBorder(20, 20, 20, 20));
		contentPane.setLayout(new BorderLayout(3, 3));
		contentPane.add(progressBar, BorderLayout.CENTER);
		setContentPane(contentPane);
	}

	/**
	 * Compie la routine di generazione dell'EXE.
	 */
	public void generate() {
		generator.setGenerationListener(new GenerationListener() {
			public void completed() {
				dispose();
			}

			public void percentual(int value) {
				progressBar.setValue(value);
			}

			public void started() {
				progressBar.setValue(0);
			}

			public void failed(Exception e) {
				exception = e;
				dispose();
			}
		});
		Thread t = new Thread(generator);
		t.start();
		setVisible(true);
	}

	/**
	 * Al termine della routine di generazione, questo metodo pu essere
	 * richiamato per verificare l'esito dell'operazione.
	 * 
	 * @return Se restituisce null, l'operazione  andata a buon fine. Se
	 *         l'operazione  fallita a causa di un'eccezione, restituisce
	 *         l'eccezione riscontrata.
	 */
	public Exception checkException() {
		return exception;
	}

}
