package it.sauronsoftware.kickstart.starter;

import java.io.File;
import java.io.FileWriter;
import java.io.FilenameFilter;
import java.io.PrintWriter;
import java.lang.reflect.Method;
import java.net.URL;
import java.net.URLClassLoader;

/**
 * Lo started dell'applicazione.
 * 
 * @author Carlo Pelliccia
 */
public class Starter {

	/**
	 * Prepara un ClassLoader per l'applicazione, capace di caricare i JAR
	 * localizzati nella cartella lib del software.
	 * 
	 * @return Il ClassLoader per l'avvio dell'applicazione.
	 * @throws Exception
	 *             In caso di problemi.
	 */
	private static ClassLoader getClassLoader() throws Exception {
		String baseDir = System.getProperty("kickstart.exe.dir");
		File libDir;
		if (baseDir != null) {
			libDir = new File(baseDir, "lib");
		} else {
			libDir = new File("lib");
		}
		File[] jars = libDir.listFiles(new FilenameFilter() {
			public boolean accept(File dir, String name) {
				return name.toLowerCase().endsWith(".jar");
			}
		});
		URL[] urls;
		if (jars != null) {
			int size = jars.length;
			urls = new URL[size];
			for (int i = 0; i < size; i++) {
				urls[i] = jars[i].toURI().toURL();
			}
		} else {
			urls = new URL[0];
		}
		return new URLClassLoader(urls);
	}

	/**
	 * Logga un'eccezione di avvio.
	 * 
	 * @param e
	 *            L'eccezione da loggare.
	 */
	private static void log(Exception e) {
		PrintWriter writer = null;
		try {
			writer = new PrintWriter(new FileWriter(new File(System
					.getProperty("user.home"), ".kickstart/starter.log"), true));
			e.printStackTrace(writer);
		} catch (Throwable t) {
			;
		} finally {
			if (writer != null) {
				try {
					writer.close();
				} catch (Throwable t) {
					;
				}
			}
		}
	}

	/**
	 * Avvia il programma.
	 * 
	 * @param args
	 *            Nessun argomento richiesto.
	 */
	public static void main(String[] args) {
		try {
			ClassLoader loader = getClassLoader();
			Class appManager = loader
					.loadClass("it.sauronsoftware.kickstart.Kickstart");
			Method startMethod = appManager.getMethod("startApplication",
					new Class[0]);
			startMethod.invoke(null, new Object[0]);
		} catch (Exception e) {
			log(e);
		}
	}

}
