package it.sauronsoftware.kickstart.util;

import java.util.Enumeration;
import java.util.Hashtable;

/**
 * Uno strumento per effettuare sostituzioni nelle stringhe.
 * 
 * @author Carlo Pelliccia
 */
public class ReplacementMap {

	/**
	 * La mappa delle sostituzioni (chiavi e valori di tipo stringa).
	 */
	private Hashtable map1 = new Hashtable();

	/**
	 * La mappa delle sostituzioni invertita (chiavi e valori di tipo stringa).
	 */
	private Hashtable map2 = new Hashtable();

	/**
	 * Aggiunge un replacement alla mappa.
	 * 
	 * @param token
	 *            Il token da sostituire.
	 * @param replacement
	 *            La sequenza con cui sostituire il token.
	 */
	public void addReplacement(String token, String replacement) {
		map1.put(token, replacement);
		map2.put(replacement, token);
	}

	/**
	 * Applica ad una stringa le sostituzioni espresse nella mappa.
	 * 
	 * @param str
	 *            La stringa originale.
	 * @return Una nuova stringa, ottenuta da quella fornita mediante le
	 *         sostituzioni espresse dalla mappa.
	 */
	public String apply(String str) {
		return apply(map1, str);
	}

	/**
	 * Applica ad una stringa le sostituzioni espresse nella mappa, in maniera
	 * inversa. Data una stringa str, il risultato della chiamata
	 * revert(apply(str))  una stringa equivalente a quella originaria.
	 * 
	 * @param str
	 *            La stringa originale.
	 * @return Una nuova stringa, ottenuta da quella fornita invertendo le
	 *         sostituzioni espresse dalla mappa.
	 */
	public String revert(String str) {
		return apply(map2, str);
	}

	/**
	 * Effettua l'elaborazione della nuova stringa ottenuta per sostituzioni.
	 * 
	 * @param map
	 *            La mappa delle sostituzioni da applicare (chiavi e valori di
	 *            tipo stringa).
	 * @param str
	 *            La stringa originale.
	 * @return Una nuova stringa ottenuta dall'originale applicando le
	 *         sostituzioni espresse nella mappa fornita.
	 */
	private String apply(Hashtable map, String str) {
		StringBuffer buffer = new StringBuffer(str);
		Enumeration tokens = map.keys();
		while (tokens.hasMoreElements()) {
			String token = (String) tokens.nextElement();
			String replacement = (String) map.get(token);
			int tokenLength = token.length();
			int replacementLength = replacement.length();
			int startAt = 0;
			while (true) {
				int index = buffer.indexOf(token, startAt);
				if (index > -1) {
					buffer.replace(index, index + tokenLength, replacement);
					startAt = index + replacementLength;
				} else {
					break;
				}
			}
		}
		return buffer.toString();
	}

}
